@@ -11,21 +11,20 @@ from pytype import state as frame_state
 from pytype import vm
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
+from pytype.tests import test_base
 from pytype.typegraph import cfg
 import six
 
 import unittest
 
 
-class AbstractTestBase(unittest.TestCase):
-
-  PYTHON_VERSION = (2, 7)
+class AbstractTestBase(test_base.UnitTest):
 
   def setUp(self):
     super(AbstractTestBase, self).setUp()
-    options = config.Options.create(python_version=self.PYTHON_VERSION)
+    options = config.Options.create(python_version=self.python_version)
     self._vm = vm.VirtualMachine(
-        errors.ErrorLog(), options, load_pytd.Loader(None, self.PYTHON_VERSION))
+        errors.ErrorLog(), options, load_pytd.Loader(None, self.python_version))
     self._program = self._vm.program
     self._node = self._vm.root_cfg_node.ConnectNew("test_node")
 
@@ -1180,6 +1179,13 @@ class AbstractTest(AbstractTestBase):
         self._vm.root_cfg_node, type_param, [{abstract_utils.K: subst_value}])
     self.assertEqual(cls, subbed_cls)
 
+  def test_singleton(self):
+    self.assertIs(abstract.Unsolvable(self._vm), abstract.Unsolvable(self._vm))
+
+  def test_singleton_subclass(self):
+    self.assertIs(abstract.Empty(self._vm), abstract.Empty(self._vm))
+    self.assertIsNot(abstract.Deleted(self._vm), abstract.Empty(self._vm))
+
 
 if __name__ == "__main__":
   unittest.main()
