@@ -10,6 +10,7 @@ from pytype import function
 from pytype import overlay
 from pytype import overlay_utils
 from pytype import utils
+from pytype.overlays import classgen
 from pytype.overlays import collections_overlay
 from pytype.pytd import pep484
 from pytype.pytd import pytd
@@ -89,8 +90,7 @@ class Callable(TypingContainer):
           self.vm.frames, inner_ellipses, args.name)
     else:
       if args.cls and args.cls.full_name == "__builtin__.list":
-        self.vm.errorlog.invalid_annotation(
-            self.vm.frames, args, "Must be constant")
+        self.vm.errorlog.ambiguous_annotation(self.vm.frames, [args])
       elif 0 not in ellipses or not isinstance(args, abstract.Unsolvable):
         self.vm.errorlog.invalid_annotation(
             self.vm.frames, args, ("First argument to Callable must be a list"
@@ -134,14 +134,11 @@ class TypeVar(abstract.PyTDFunction):
 
   def _get_annotation(self, node, var, name):
     with self.vm.errorlog.checkpoint() as record:
-      retvar = self.vm.annotations_util.process_annotation_var(
+      annot = self.vm.annotations_util.extract_annotation(
           node, var, name, self.vm.simple_stack())
     if record.errors:
       raise TypeVarError("\n".join(error.message for error in record.errors))
-    try:
-      return abstract_utils.get_atomic_value(retvar)
-    except abstract_utils.ConversionError:
-      raise TypeVarError("%s must be constant" % name)
+    return annot
 
   def _get_namedarg(self, node, args, name, default_value):
     if name not in args.namedargs:
@@ -203,28 +200,22 @@ class Cast(abstract.PyTDFunction):
 
   def call(self, node, func, args):
     if args.posargs:
-      annot = self.vm.annotations_util.process_annotation_var(
+      annot = self.vm.annotations_util.extract_annotation(
           node, args.posargs[0], "typing.cast", self.vm.simple_stack())
-      if any(t.formal for t in annot.data):
+      if annot.formal:
         self.vm.errorlog.invalid_typevar(
             self.vm.frames, "cannot pass a TypeVar to a function")
         return node, self.vm.new_unsolvable(node)
-      return node, self.vm.annotations_util.init_annotation_var(
-          node, "typing.cast", annot)
+      return self.vm.init_class(node, annot)
     return super(Cast, self).call(node, func, args)
 
 
-class NoReturn(abstract.AtomicAbstractValue):
+class NoReturn(abstract.Singleton):
+  """Implements typing.NoReturn as a singleton."""
 
   def __init__(self, vm):
     super(NoReturn, self).__init__("NoReturn", vm)
 
-  def get_class(self):
-    return self
-
-  def compute_mro(self):
-    return self.default_mro()
-
 
 def build_any(name, vm):
   del name
@@ -521,25 +512,30 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
     return self.namedtuple.call(node, None, args)
 
   def make_class(self, node, f_locals):
+    # If BuildClass.call() hits max depth, f_locals will be [unsolvable]
+    # Since we don't support defining NamedTuple subclasses in a nested scope
+    # anyway, we can just return unsolvable here to prevent a crash, and let the
+    # invalid namedtuple error get raised later.
+    if f_locals.data[0].isinstance_Unsolvable():
+      return node, self.vm.new_unsolvable(node)
+
     f_locals = abstract_utils.get_atomic_python_constant(f_locals)
 
     # retrieve __qualname__ to get the name of class
     name = f_locals["__qualname__"]
-    # retrieve __annotations__ to get the dict
-    # with key-value pair of (variable, type)
-    anno = f_locals.get("__annotations__", {})
-    if anno:
-      anno = abstract_utils.get_atomic_value(anno)
 
     # assemble the arguments that are compatible with NamedTupleFuncBuilder.call
     field_list = []
     defaults = []
-    for k, v in anno.items():
+    cls_locals = classgen.get_class_locals(
+        abstract_utils.get_atomic_python_constant(name), allow_methods=True,
+        ordering=classgen.Ordering.FIRST_ANNOTATE, vm=self.vm)
+    for k, local in cls_locals.items():
+      assert local.typ
       if k in f_locals:
-        defaults.append(f_locals.get(k))
-        # TODO(ahxun): check if the value matches the declared type
+        defaults.append(f_locals[k])
       k = self.vm.convert.constant_to_var(k, node=node)
-      field_list.append(self.vm.convert.build_tuple(node, (k, v)))
+      field_list.append(self.vm.convert.build_tuple(node, (k, local.typ)))
     anno = self.vm.convert.build_list(node, field_list)
     posargs = (name, anno)
     args = function.Args(posargs=posargs)
@@ -647,7 +643,7 @@ class Generic(TypingContainer):
       inner = [item for item in inner
                if isinstance(item, abstract.TypeParameter)]
 
-    template = [item.full_name for item in inner]
+    template = [item.name for item in inner]
 
     if len(set(template)) != len(template):
       self.vm.errorlog.invalid_annotation(
