@@ -374,6 +374,33 @@ class ParserTest(_ParserTestBase):
 
       x: somewhere.Foo""")
 
+  def test_external_alias(self):
+    self.check("""
+      from somewhere import Foo
+
+      class Bar:
+          Baz = Foo
+    """, """
+      from typing import Any
+
+      from somewhere import Foo
+
+      class Bar:
+          Baz: Any
+    """)
+
+  def test_same_named_alias(self):
+    self.check("""
+      import somewhere
+      class Bar:
+          Foo = somewhere.Foo
+    """, """
+      from typing import Any
+
+      class Bar:
+          Foo: Any
+    """)
+
   def test_type_params(self):
     ast = self.check("""
       from typing import TypeVar
@@ -1256,10 +1283,48 @@ class ClassTest(_ParserTestBase):
       """)
     self.check_error("""
       class Foo(badkeyword=Meta): ...
-      """, 1, "Only 'metaclass' allowed as classdef kwarg")
+      """, 1, "Unexpected classdef kwarg 'badkeyword'")
     self.check_error("""
       class Foo(metaclass=Meta, Bar): ...
-      """, 1, "metaclass must be last argument")
+      """, 1, "non-keyword arguments cannot follow keyword arguments")
+
+  def test_typed_dict_kwarg(self):
+    self.check("""
+      from typing import TypedDict
+
+      class Foo(TypedDict, total=False): ...
+    """, """
+      from typing import TypedDict
+
+      class Foo(TypedDict): ...
+    """)
+    self.check_error("""
+      class Foo(object, total=False): ...
+    """, 1, "'total' allowed as classdef kwarg only for TypedDict subclasses")
+
+  def test_typing_extensions_typed_dict(self):
+    self.check("""
+      from typing_extensions import TypedDict
+
+      class Foo(TypedDict, total=False): ...
+    """, """
+      import typing_extensions
+
+      from typing_extensions import TypedDict
+
+      class Foo(typing_extensions.TypedDict): ...
+    """)
+
+  def test_multiple_classdef_kwargs(self):
+    self.check("""
+      from typing import TypedDict
+
+      class Foo(TypedDict, total=False, metaclass=Meta): ...
+    """, """
+      from typing import TypedDict
+
+      class Foo(TypedDict, metaclass=Meta): ...
+    """)
 
   def test_shadow_pep484(self):
     self.check("""
@@ -1612,13 +1677,6 @@ class ClassIfTest(_ParserTestBase):
           import foo
     """, 3, "syntax error")
 
-  def test_bad_alias(self):
-    self.check_error("""
-      class Foo:
-        if sys.version_info > (3, 4, 0):
-          a = b
-    """, 1, "Illegal value for alias 'a'")
-
   def test_no_class(self):
     self.check("""
       class Foo:
